@@ -264,10 +264,8 @@ struct sshfs {
 	int server_version;
 	unsigned remote_uid;
 	unsigned local_uid;
-#ifdef __APPLE__
 	unsigned remote_gid;
 	unsigned local_gid;
-#endif
 	int remote_uid_detected;
 	unsigned blksize;
 	char *progname;
@@ -789,17 +787,12 @@ static int buf_get_attrs(struct buffer *buf, struct stat *stbuf, int *flagsp)
 		}
 	}
 
-#ifdef __APPLE__
 	if (sshfs.remote_uid_detected) {
 		if (uid == sshfs.remote_uid)
 			uid = sshfs.local_uid;
 		if (gid == sshfs.remote_gid)
 			gid = sshfs.local_gid;
 	}
-#else /* !__APPLE__ */
-	if (sshfs.remote_uid_detected && uid == sshfs.remote_uid)
-		uid = sshfs.local_uid;
-#endif /* __APPLE__ */
 	if (sshfs.idmap == IDMAP_FILE && sshfs.uid_map)
 		if (translate_id(&uid, sshfs.uid_map) == -1)
 			return -EPERM;
@@ -1296,9 +1289,10 @@ static void chunk_put_locked(struct read_chunk *chunk)
 	pthread_mutex_unlock(&sshfs.lock);
 }
 
-static int clean_req(void *key_, struct request *req)
+static int clean_req(void *key_, struct request *req, gpointer user_data_)
 {
 	(void) key_;
+	(void) user_data_;
 
 	req->error = -EIO;
 	if (req->want_reply)
@@ -1600,10 +1594,8 @@ static void sftp_detect_uid()
 
 	sshfs.remote_uid = stbuf.st_uid;
 	sshfs.local_uid = getuid();
-#ifdef __APPLE__
 	sshfs.remote_gid = stbuf.st_gid;
 	sshfs.local_gid = getgid();
-#endif
 	sshfs.remote_uid_detected = 1;
 	DEBUG("remote_uid = %i\n", sshfs.remote_uid);
 
@@ -2424,17 +2416,12 @@ static int sshfs_chown(const char *path, uid_t uid, gid_t gid,
 			return -EIO;
 	}
 	
-#ifdef __APPLE__
 	if (sshfs.remote_uid_detected) {
 		if (uid == sshfs.local_uid)
 			uid = sshfs.remote_uid;
 		if (gid == sshfs.local_gid)
 			gid = sshfs.remote_gid;
 	}
-#else /* !__APPLE__ */
-	if (sshfs.remote_uid_detected && uid == sshfs.local_uid)
-		uid = sshfs.remote_uid;
-#endif /* __APPLE__ */
 	if (sshfs.idmap == IDMAP_FILE && sshfs.r_uid_map)
 		if(translate_id(&uid, sshfs.r_uid_map) == -1)
 			return -EPERM;
@@ -3484,7 +3471,19 @@ static int sshfs_opt_proc(void *data, const char *arg, int key,
 				}
 			}
 #else
-			sshfs.mountpoint = realpath(arg, NULL);
+                        int fd, len;
+                        if (sscanf(arg, "/dev/fd/%u%n", &fd, &len) == 1 &&
+                            len == strlen(arg)) {
+                                /*
+                                 * Allow /dev/fd/N unchanged; it can be
+                                 * use for pre-mounting a generic fuse
+                                 * mountpoint to later be completely
+                                 * unprivileged with libfuse >= 3.3.0.
+                                 */
+                                sshfs.mountpoint = arg;
+                        } else {
+                                sshfs.mountpoint = realpath(arg, NULL);
+                        }
 #endif
 			if (!sshfs.mountpoint) {
 				fprintf(stderr, "sshfs: bad mount point `%s': %s\n",
@@ -3597,40 +3596,60 @@ static int read_password(void)
 	return 0;
 }
 
+// Behaves similarly to strtok(), but allows for the ' ' delimiter to be escaped
+// by '\ '.
+static char *tokenize_on_space(char *str)
+{
+	static char *pos = NULL;
+	char *start = NULL;
+
+	if (str)
+		pos = str;
+
+	if (!pos)
+		return NULL;
+
+	// trim any leading spaces
+	while (*pos == ' ')
+		pos++;
+
+	start = pos;
+
+	while (pos && *pos != '\0') {
+		// break on space, but not on '\ '
+		if (*pos == ' ' && *(pos - 1) != '\\') {
+			break;
+		}
+		pos++;
+	}
+
+	if (*pos == '\0') {
+		pos = NULL;
+	}
+	else {
+		*pos = '\0';
+		pos++;
+	}
+
+	return start;
+}
+
 static void set_ssh_command(void)
 {
-	char *s;
-	char *d;
+	char *token = NULL;
 	int i = 0;
-	int end = 0;
 
-	d = sshfs.ssh_command;
-	s = sshfs.ssh_command;
-	while (!end) {
-		switch (*s) {
-		case '\0':
-			end = 1;
-		case ' ':
-			*d = '\0';
-			if (i == 0) {
-				replace_arg(&sshfs.ssh_args.argv[0],
-					    sshfs.ssh_command);
-			} else {
-				if (fuse_opt_insert_arg(&sshfs.ssh_args, i,
-						sshfs.ssh_command) == -1)
-					_exit(1);
-			}
-			i++;
-			d = sshfs.ssh_command;
-			break;
-
-		case '\\':
-			if (s[1])
-				s++;
-		default:
-			*d++ = *s;
+	token = tokenize_on_space(sshfs.ssh_command);
+	while (token != NULL) {
+		if (i == 0) {
+			replace_arg(&sshfs.ssh_args.argv[0], token);
+		} else {
+			if (fuse_opt_insert_arg(&sshfs.ssh_args, i, token) == -1)
+				_exit(1);
 		}
-		s++;
+		i++;
+
+		token = tokenize_on_space(NULL);
 	}
 }
 
